@@ -1,46 +1,55 @@
 /*
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-*/
-
-// written by Jon Keating <jon@licq.org>
+ * This file is part of Licq, an instant messaging client for UNIX.
+ * Copyright (C) 2004-2010 Licq developers
+ *
+ * Licq is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Licq is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Licq; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 
 #ifndef __MSNPACKET_H
 #define __MSNPACKET_H
 
-#include "licq_packets.h"
+#include <cstdlib>
+#include <pthread.h>
+#include <string>
+
+#include <licq/packet.h>
 
 #include "msnbuffer.h"
 
-#include <pthread.h>
-#include <stdlib.h>
+static const std::string base64_chars =
+             "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+             "abcdefghijklmnopqrstuvwxyz"
+             "0123456789+/";
 
-class CMSNPacket : public CPacket
+class CMSNPacket : public Licq::Packet
 {
 public:
   CMSNPacket(bool = false);
   virtual ~CMSNPacket() { if (m_pBuffer) delete m_pBuffer; if (m_szCommand) free(m_szCommand); }
   CMSNBuffer *getBuffer() { return m_pBuffer; }
   
-  const unsigned short Command() { return 0; }
-  const unsigned short SubSequence() { return 0; }
-  const unsigned short SubCommand() { return 0; }
-  const unsigned short Sequence() { return m_nSequence; }
+  unsigned short Command() { return 0; }
+  unsigned short SubSequence() { return 0; }
+  unsigned short SubCommand() { return 0; }
+  unsigned short Sequence() { return m_nSequence; }
   const char* MSNCommand() { return m_szCommand; }
   
   virtual void InitBuffer();
-  
+
+  char *CreateGUID();
+
 protected:
   
   CMSNBuffer *m_pBuffer;
@@ -60,14 +69,47 @@ public:
   CMSNPayloadPacket(bool);
   virtual ~CMSNPayloadPacket() { }
   
-  void InitBuffer();
+  virtual void InitBuffer();
   
 protected:
-
   unsigned long m_nPayloadSize;
   bool m_bAck;
 };
 
+class CMSNP2PPacket : public CMSNPayloadPacket
+{
+public:
+  CMSNP2PPacket(const char *,unsigned long = 0, unsigned long = 0, unsigned long = 0,
+		unsigned long = 0, unsigned long = 0, unsigned long = 0,
+		unsigned long = 0, unsigned long = 0, unsigned long = 0,
+		unsigned long = 0, unsigned long = 0, unsigned long = 0);
+  virtual ~CMSNP2PPacket();
+
+  void InitBuffer();
+
+  unsigned long SessionId() { return m_nSessionId; }
+  unsigned long BaseId() { return m_nBaseId; }
+  char *CallGUID() { return m_szCallGUID; }
+  // unsigned long DataSize() {}
+  //unsigned long Offset() {}
+
+protected:
+  char *m_szToEmail,
+       *m_szCallGUID;
+  unsigned long m_nSessionId,
+    m_nBaseId,
+    m_nDataSizeLO,
+    m_nDataSizeHI,
+    m_nDataOffsetLO,
+    m_nDataOffsetHI,
+    m_nLen,
+    m_nFlag,
+    m_nAckId,
+    m_nAckUniqueId,
+    m_nAckDataLO,
+    m_nAckDataHI;
+};
+
 class CPS_MSNVersion : public CMSNPacket
 {
 public:
@@ -103,7 +145,7 @@ public:
 class CPS_MSNAuthenticate : public CMSNPacket
 {
 public:
-  CPS_MSNAuthenticate(char *, char *, const char *);
+  CPS_MSNAuthenticate(char *, const std::string& password, const char *);
   virtual ~CPS_MSNAuthenticate() { if (m_szCookie) free(m_szCookie); }
   
 protected:
@@ -123,7 +165,7 @@ protected:
 class CPS_MSNChangeStatus : public CMSNPacket
 {
 public:
-  CPS_MSNChangeStatus(unsigned long);
+  CPS_MSNChangeStatus(std::string& status);
 };
 
 class CPS_MSNLogoff : public CMSNPacket
@@ -236,7 +278,7 @@ public:
 class CPS_MSNCall : public CMSNPacket
 {
 public:
-  CPS_MSNCall(char *);
+  CPS_MSNCall(const char*);
   virtual ~CPS_MSNCall() { if (m_szUser) free(m_szUser); }
   
 protected:
@@ -249,6 +291,35 @@ public:
   CPS_MSNTypingNotification(const char *);
 };
 
+class CPS_MSNCancelInvite : public CMSNPayloadPacket
+{
+public:
+  CPS_MSNCancelInvite(const std::string& cookie, const std::string& code);
+};
+
+class CPS_MSNInvitation : public CMSNP2PPacket
+{
+public:
+  CPS_MSNInvitation(const char* szToEmail, const char* szFromEmail,
+      const char* szMSNObject);
+};
+
+class CPS_MSNP2PBye : public CMSNP2PPacket
+{
+public:
+  CPS_MSNP2PBye(const char *szId, const char *szFromId, const char *szCallId,
+                unsigned long nBaseId, unsigned long nAckId,
+		unsigned long nDataSizeHI, unsigned long nDataSizeLO);
+};
+
+class CPS_MSNP2PAck : public CMSNP2PPacket
+{
+public:
+  CPS_MSNP2PAck(const char *szId, unsigned long nSessionId,
+		unsigned long nBaseId, unsigned long nAckId,
+		unsigned long nAckBaseId, unsigned long nDataSizeHI,
+		unsigned long nDataSizeLO);
+};
 
 #endif // __MSNPACKET_H
 
